import std.stdio;
import std.getopt;
import std.string;

import utils;
import file;
import config;
import desktopfile;
import menugenerator;


string VERSION = "0.2beta";

/*
 * Command Line Flags
 */
bool showHelp, menuXML, pipeMenuXML, pipeMenu, windowsAndDesktopMenu;


int main(string[] argv)
{
    try
    {
        parseCmdLine(argv);
        
        if (showHelp) writefln(usage());
        
        else
        {
            Config cfg = new Config();
            
            cfg.showWindowsAndDesktopMenu(windowsAndDesktopMenu);
            
            DesktopFileParser dfParser = new DesktopFileParser(cfg);
        
            string[string][string] infoList;
            
                        
            foreach(file; utils.getDotDesktopFiles(cfg.DOT_DESKTOP_LOCATIONS))
            {
                string[string] info = dfParser.parse(file);
                                
                if (info !is null)
                    infoList[info["Name"]~info["Exec"]] = info;
            }
        
            MenuGenerator mg = new MenuGenerator(infoList, cfg, argv);
            
            if (menuXML) File.Write(cfg.MENUXML, mg.getStaticMenuXML());
    	    else if (pipeMenuXML) File.Write(cfg.MENUXML, mg.getPipeMenuXML());
    	    else if (pipeMenu) writefln(mg.getPipeMenu());
        }
    }
    catch (Exception e)
    {
        writefln(e.msg);
        return 1;
    }
    return 0;
}


/*
 * Prints help
 */
string usage()
{
    string help = format("OpenBox3 Menu Generator (using .desktop files) v%s (GPLv2)\n", VERSION);
    help ~= "Copyright (c) 2008 Pável Varela Rodríguez <neonskull@gmail.com>\n"
    "\nUsage: ob3menugen [ACTION] [OPTIONS]\n"
    "\nACTIONS:\n"
    "    NO_ACTION         Generates menu.xml, with static content.\n\n"
    "    -p                Generates menu.xml, with needed intructions to\n"
    "                      generate a pipe menu.\n\n"
    "    --pipe            Generates the pipe menu.\n"
    "                      Usually you will not use this option directly.\n\n"
    "    -h|--help         This help.\n"
    "\nOPTIONS:\n"
    "    -w|--windows      Adds the Windows (desktops and running apps) menuitem.\n"
    "\nExamples:\n"
    "    To simply generate an static menu:\n"
    "        $ ob3menugen\n\n"
    "    The same as above, but with the \"Windows\" menuitem:\n"
    "        $ ob3menugen -w\n\n"
    "    To generate a menu file, with instructions to use a pipe menu and\n"
    "    to include the \"Windows\" menuitem:\n"
    "        $ ob3menugen -p -w\n\n";
    help ~= format("Compilation date: %s", __TIMESTAMP__);
    return help;
}


void parseCmdLine(string[] argv)
{
    try
    {
        getopt(argv,
               std.getopt.config.noBundling,
               std.getopt.config.stopOnFirstNonOption,
               "h|help", &showHelp,
               "p", &pipeMenuXML,
               "pipe", &pipeMenu,
               "w|windows", &windowsAndDesktopMenu
               );
        if (argv.length > 1)
            foreach(arg; argv[1..length])
                if (!startsWith(arg, "-"))
                    throw new Exception(format("Unrecognized option '%s'", arg));
        
    	if ((!pipeMenuXML) && (!pipeMenu)) menuXML = true;
    }
    catch (Exception e)
    {
        string optionsHelp()
        {
            string str = std.string.join(splitlines(usage())[2..length-1], "\n");
            return str[1..length]; 
        }
        
        string msg = format("Error: %s\n%s", e.msg, optionsHelp());
        throw new Exception(msg);
    }
}

